@@ -3,7 +3,7 @@
 ###########################################################################
 #    OCRFeeder - The complete OCR suite
 #    Copyright (C) 2009 Joaquim Rocha
-# 
+#
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
 #    the Free Software Foundation, either version 3 of the License, or
@@ -43,14 +43,14 @@ PREDEFINED_ENGINES = {'tesseract': {'name': 'Tesseract',
                      }
 
 class ProjectSaver:
-    
+
     def __init__(self, pages_data, temp_dir = '/tmp'):
-        
+
         self.pages_data = pages_data
         self.document = minidom.Document()
         self.images = {}
         self.temp_dir = temp_dir
-    
+
     def __handleImageEmbedding(self, page_data):
         base_name = os.path.basename(page_data.image_path)
         embedded_names = []
@@ -64,7 +64,7 @@ class ProjectSaver:
             i += 1
         self.images[page_data.image_path] = base_name
         return base_name
-    
+
     def __imagesToXml(self, root_node):
         for page_data in self.pages_data:
             self.__handleImageEmbedding(page_data)
@@ -78,7 +78,7 @@ class ProjectSaver:
             new_image.appendChild(original)
             new_image.appendChild(embedded)
             images_node.appendChild(new_image)
-    
+
     def convertToXml(self, item, root_node):
         if type(item) == dict:
             for key, value in item.items():
@@ -92,14 +92,14 @@ class ProjectSaver:
             text_node = self.document.createTextNode(str(item))
             root_node.appendChild(text_node)
         return root_node
-    
+
     def serialize(self, file_name):
         root_node = self.document.createElement('ocrfeeder')
         pages_dict = {'pages': [page_data.convertToDict() for page_data in self.pages_data]}
         new_node = self.convertToXml(pages_dict, root_node)
         self.__imagesToXml(root_node)
         self.__createProjectFile(new_node.toxml(), file_name)
-    
+
     def __createProjectFile(self, xml_content, file_name):
         temp_dir = tempfile.mkstemp(dir = self.temp_dir)[1]
         try:
@@ -123,16 +123,16 @@ class ProjectSaver:
         zip.close()
         os.chdir(old_dir)
         shutil.rmtree(temp_dir, ignore_errors = True)
-    
+
 class ProjectLoader:
-    
+
     def __init__(self, project_file, temp_dir = '/tmp'):
         self.temp_dir = temp_dir
         if not (os.path.isfile(project_file) and project_file.endswith('.ocrf')):
             #raise
             pass
         self.configuration_dir = self.unzipFile(project_file)
-    
+
     def loadConfiguration(self, folder = None):
         folder = folder or self.configuration_dir
         project_xml = os.path.join(folder, 'project.xml')
@@ -150,7 +150,7 @@ class ProjectLoader:
             for data_box in page_data['data_boxes']:
                 args = []
                 # text variable is to avoid problems with
-                # escaping characters 
+                # escaping characters
                 text = ''
                 for var_name, value in data_box.items():
                     if var_name == 'text':
@@ -171,7 +171,7 @@ class ProjectLoader:
             page = PageData(image_path, data_boxes)
             pages.append(page)
         return pages
-    
+
     def __getImagesInfo(self, images_nodes_list):
         images = {}
         for image_node in images_nodes_list:
@@ -185,7 +185,7 @@ class ProjectLoader:
             if original_name and embedded_name:
                 images[original_name] = embedded_name
         return images
-    
+
     def __getPageDataInfo(self, pages_nodes_list):
         page_data_info = []
         for page in pages_nodes_list:
@@ -198,14 +198,14 @@ class ProjectLoader:
                     page_data[child.localName] = child.childNodes[0].nodeValue
             page_data_info.append(page_data)
         return page_data_info
-    
+
     def __getDataBoxesInfo(self, data_boxes_node):
         data_boxes_info = []
         for child in data_boxes_node.childNodes:
             if child.localName == 'DataBox':
                 data_boxes_info.append(self.__getDataBoxInfo(child))
         return data_boxes_info
-    
+
     def __getDataBoxInfo(self, data_box_node):
         data_box_info = {}
         for child in data_box_node.childNodes:
@@ -218,20 +218,20 @@ class ProjectLoader:
                 else:
                     data_box_info[child.localName] = ''
         return data_box_info
-    
+
     def __getTextDatasInfo(self, text_datas):
         text_datas_info = []
         for child in text_datas.childNodes:
             if child.localName == 'TextData':
                 text_datas_info.append(self.__getTextDataInfo(child))
         return text_datas_info
-    
+
     def __getTextDataInfo(self, text_data):
         text_data_info = {}
         for child in text_data.childNodes:
             text_data_info[child.localName] = child.childNodes[0].nodeValue
         return text_data_info
-    
+
     def unzipFile(self, file):
         base_name = os.path.basename(file)
         export_dir = os.path.join(self.temp_dir, os.path.splitext(base_name)[0])
@@ -253,24 +253,55 @@ class ProjectLoader:
                 outfile.close()
         return export_dir
 
-class ConfigurationManager:
-    
-    DEFAULTS = {'temporary_dir': '/tmp',
-                'text_fill': '94, 156, 235, 150',
-                'text_stroke': '94, 156, 235, 250',
-                'image_fill': '0, 183, 0, 150',
-                'image_stroke': '94, 156, 235, 250',
-                'window_size': 'auto',
-                'unpaper': '/usr/bin/unpaper',
-                'favorite_engine': 'ocrad'
+class ConfigurationManager(object):
+
+    TEMPORARY_DIR = 'temporary_dir'
+    TEXT_FILL = 'text_fill'
+    IMAGE_FILL = 'image_fill'
+    BOXES_STROKE = 'boxes_stroke'
+    WINDOW_SIZE = 'window_size'
+    UNPAPER = 'unpaper'
+    UNPAPER_USE_BLACK_FILTER = 'unpaper_use_black_filter'
+    UNPAPER_NOISE_FILTER_INTENSITY = 'unpaper_noise_filter_intensity'
+    UNPAPER_GRAY_FILTER_SIZE = 'unpaper_gray_filter_size'
+    UNPAPER_EXTRA_OPTIONS = 'unpaper_extra_options'
+    UNPAPER_IMAGES_AFTER_ADDITION = 'unpaper_images_after_addition'
+    FAVORITE_ENGINE = 'favorite_engine'
+    IMPROVE_COLUMN_DETECTION = 'improve_column_detection'
+    COLUMN_MIN_WIDTH = 'column_min_width'
+    CLEAN_TEXT = 'clean_text'
+    ADJUST_BOXES_BOUNDS = 'adjust_boxes_bounds'
+    BOUNDS_ADJUSTMENT_SIZE = 'bounds_adjustment_size'
+    DESKEW_IMAGES_AFTER_ADDITION = 'deskew_images_after_addition'
+
+    DEFAULTS = {TEMPORARY_DIR: '/tmp',
+                TEXT_FILL: (94, 156, 235, 150),
+                BOXES_STROKE: (94, 156, 235, 250),
+                IMAGE_FILL: (0, 183, 0, 150),
+                WINDOW_SIZE: 'auto',
+                UNPAPER: getExecPath('unpaper'),
+                UNPAPER_USE_BLACK_FILTER: True,
+                UNPAPER_NOISE_FILTER_INTENSITY: 'auto',
+                UNPAPER_GRAY_FILTER_SIZE: 'auto',
+                FAVORITE_ENGINE: 'ocrad',
+                IMPROVE_COLUMN_DETECTION: True,
+                COLUMN_MIN_WIDTH: 'auto',
+                CLEAN_TEXT: True,
+                ADJUST_BOXES_BOUNDS: True,
+                BOUNDS_ADJUSTMENT_SIZE: 'auto',
+                DESKEW_IMAGES_AFTER_ADDITION: True,
+                UNPAPER_IMAGES_AFTER_ADDITION: True,
+                UNPAPER_EXTRA_OPTIONS: '',
                 }
-    
+
+    conf = dict(DEFAULTS)
+
     def __init__(self):
-        self.setDefaults()
         self.user_configuration_folder = os.path.expanduser('~/.ocrfeeder')
         self.user_engines_folder = os.path.join(self.user_configuration_folder, 'engines')
         self.makeUserConfigurationFolder()
-            
+        self.has_unpaper = self.getDefault(self.UNPAPER)
+
     def makeUserConfigurationFolder(self):
         if not os.path.exists(self.user_engines_folder):
             os.makedirs(self.user_engines_folder)
@@ -301,66 +332,190 @@ class ConfigurationManager:
                             failure_string = failure_string)
             existing_engines.append(engine)
         return existing_engines
-    
+
+    def setConf(self, conf_key, value):
+        ConfigurationManager.conf[conf_key] = value
+
+    def getConf(self, conf_key):
+        return ConfigurationManager.conf[conf_key]
+
     def setTemporaryDir(self, temp_dir):
-        self.temporary_dir = temp_dir
-    
+        self.setConf(self.TEMPORARY_DIR, temp_dir)
+
+    def setTemporaryDir(self, temp_dir):
+        self.setConf(self.TEMPORARY_DIR, temp_dir)
+
     def getTemporaryDir(self):
-        return self.temporary_dir
-    
+        return self.getConf(self.TEMPORARY_DIR)
+
     def setFavoriteEngine(self, engine_name):
-        self.favorite_engine = engine_name
-    
+        self.setConf(self.FAVORITE_ENGINE, engine_name)
+
     def getFavoriteEngine(self):
-        return self.favoriteEngine
-    
-    def __getColorString(self, color):
-        string_color = ', '.join([str(color_value) for color_value in list(color)])
-        return string_color
-    
+        return self.getConf(self.FAVORITE_ENGINE)
+
+    def __getColorFromString(self, color):
+        if type(color) != str:
+            return color
+        color_list = [value.strip('()\ ') for value in color.split(',')]
+        try:
+            int_color_list = [int(value) for value in color_list]
+        except ValueError, exception:
+            return None
+        return tuple(int_color_list)
+
     def setTextFill(self, color):
-        self.text_fill = self.__getColorString(color)
-    
-    def setTextStroke(self, color):
-        self.text_stroke = self.__getColorString(color)
-    
+        self.setConf(self.TEXT_FILL, color)
+
+    def setBoxesStroke(self, color):
+        self.setConf(self.BOXES_STROKE, color)
+
     def setImageFill(self, color):
-        self.image_fill = self.__getColorString(color)
-    
-    def setImageStroke(self, color):
-        self.image_stroke = self.__getColorString(color)
-    
+        self.setConf(self.IMAGE_FILL, color)
+
     def getTextFill(self):
-        return eval(self.text_fill)
-    
-    def getTextStroke(self):
-        return eval(self.text_stroke)
-    
+        return self.__getColorFromString(self.getConf(self.TEXT_FILL))
+
+    def getBoxesStroke(self):
+        return self.__getColorFromString(self.getConf(self.BOXES_STROKE))
+
     def getImageFill(self):
-        return eval(self.image_fill)
-    
-    def getImageStroke(self):
-        return eval(self.image_stroke)
-    
+        return self.__getColorFromString(self.getConf(self.IMAGE_FILL))
+
     def setWindowSize(self, window_size):
-        self.window_size = window_size
-    
+        self.setConf(self.WINDOW_SIZE, window_size)
+
+    def getWindowSize(self):
+        return self.getConf(self.WINDOW_SIZE)
+
     def setUnpaper(self, unpaper):
-        self.unpaper = unpaper
-    
+        self.setConf(self.UNPAPER, unpaper)
+
     def getUnpaper(self):
-        return self.unpaper
-    
+        return self.getConf(self.UNPAPER)
+
+    def setUseBlackFilter(self, use_black_filter):
+        self.setConf(self.UNPAPER_USE_BLACK_FILTER, use_black_filter)
+
+    def getUseBlackFilter(self):
+        use_black_filter = self.getConf(self.UNPAPER_USE_BLACK_FILTER)
+        return self.__convertBoolSetting(use_black_filter)
+
+    def setNoiseFilterIntensity(self, noise_filter_intensity):
+        self.setConf(self.UNPAPER_NOISE_FILTER_INTENSITY,
+                     noise_filter_intensity)
+
+    def getNoiseFilterIntensity(self):
+        noise_filter_intensity = \
+            self.getConf(self.UNPAPER_NOISE_FILTER_INTENSITY)
+        if noise_filter_intensity == 'auto' or noise_filter_intensity == 'none':
+            return noise_filter_intensity
+        try:
+            noise_filter_intensity_int = int(noise_filter_intensity)
+        except ValueError:
+            return 'auto'
+        return noise_filter_intensity_int
+
+    def setGrayFilterSize(self, gray_filter_size):
+        self.setConf(self.UNPAPER_GRAY_FILTER_SIZE,
+                     gray_filter_size)
+
+    def getGrayFilterSize(self):
+        gray_filter_size = self.getConf(self.UNPAPER_GRAY_FILTER_SIZE)
+        if gray_filter_size == 'auto' or gray_filter_size == 'none':
+            return gray_filter_size
+        try:
+            gray_filter_size_int = int(gray_filter_size)
+        except ValueError:
+            return 'auto'
+        return gray_filter_size_int
+
+    def setUnpaperExtraOptions(self, extra_options):
+        self.setConf(self.UNPAPER_EXTRA_OPTIONS, extra_options)
+
+    def getUnpaperExtraOptions(self):
+        return self.getConf(self.UNPAPER_EXTRA_OPTIONS)
+
+    def setUnpaperImagesAfterAddition(self, unpaper_images_after_addition):
+        self.setConf(self.UNPAPER_IMAGES_AFTER_ADDITION,
+                     unpaper_images_after_addition)
+
+    def getUnpaperImagesAfterAddition(self):
+        unpaper = self.getConf(self.UNPAPER_IMAGES_AFTER_ADDITION)
+        return self.__convertBoolSetting(unpaper)
+
+    def setImproveColumnDetection(self, improve_column_detection):
+        self.setConf(self.IMPROVE_COLUMN_DETECTION, improve_column_detection)
+
+    def getImproveColumnDetection(self):
+        improve = self.getConf(self.IMPROVE_COLUMN_DETECTION)
+        return self.__convertBoolSetting(improve)
+
+    def setColumnMinWidth(self, column_min_width):
+        self.setConf(self.COLUMN_MIN_WIDTH, column_min_width)
+
+    def getColumnMinWidth(self):
+        column_min_width = self.getConf(self.COLUMN_MIN_WIDTH)
+        if column_min_width == 'auto':
+            return column_min_width
+        try:
+            column_min_width_int = int(column_min_width)
+        except ValueError:
+            return 'auto'
+        return column_min_width_int
+
+    def getCleanText(self):
+        clean_text = self.getConf(self.CLEAN_TEXT)
+        return self.__convertBoolSetting(clean_text)
+
+    def setCleanText(self, clean_text):
+        self.setConf(self.CLEAN_TEXT, clean_text)
+
+    def setAdjustBoxesBounds(self, adjust_boxes_bounds):
+        self.setConf(self.ADJUST_BOXES_BOUNDS, adjust_boxes_bounds)
+
+    def getAdjustBoxesBounds(self):
+        adjust = self.getConf(self.ADJUST_BOXES_BOUNDS)
+        return self.__convertBoolSetting(adjust)
+
+    def setDeskewImagesAfterAddition(self, deskew_images_after_addition):
+        self.setConf(self.DESKEW_IMAGES_AFTER_ADDITION,
+                     deskew_images_after_addition)
+
+    def getDeskewImagesAfterAddition(self):
+        deskew = self.getConf(self.DESKEW_IMAGES_AFTER_ADDITION)
+        return self.__convertBoolSetting(deskew)
+
+    def setBoundsAdjustmentSize(self, adjustment_size):
+        self.setConf(self.BOUNDS_ADJUSTMENT_SIZE, adjustment_size)
+
+    def getBoundsAdjustmentSize(self):
+        adjustment_size = self.getConf(self.BOUNDS_ADJUSTMENT_SIZE)
+        if adjustment_size == 'auto':
+            return adjustment_size
+        try:
+            adjustment_size_int = int(adjustment_size)
+        except ValueError:
+            return 'auto'
+        return adjustment_size_int
+
+    def __convertBoolSetting(self, setting):
+        if type(setting) == str:
+            if setting == 'True':
+                setting = True
+            else:
+                setting = False
+        return setting
+
     def setDefaults(self):
-        for key, value in self.DEFAULTS.items():
-            exec('self.%s = "%s"' % (key, value))
-    
+        ConfigurationManager.conf = dict(self.DEFAULTS)
+
     def getDefault(self, variable_name):
         if variable_name in self.DEFAULTS.keys():
             return self.DEFAULTS[variable_name]
         else:
             return ''
-    
+
     def loadConfiguration(self):
         configuration_file = os.path.join(self.user_configuration_folder, 'preferences.xml')
         if not os.path.isfile(configuration_file):
@@ -372,20 +527,67 @@ class ConfigurationManager:
                 for node in nodeList:
                     for child in node.childNodes:
                         if child.nodeType == child.TEXT_NODE:
-                            exec('self.%s = "%s"' % (key, child.nodeValue))
+                            ConfigurationManager.conf[key] = str(child.nodeValue)
                             break
         return True
-    
+
     def configurationToXml(self):
         configuration_file = os.path.join(self.user_configuration_folder, 'preferences.xml')
         doc = minidom.Document()
         root_node = doc.createElement('ocrfeeder')
-        for key in self.DEFAULTS.keys():
+        for key, value in ConfigurationManager.conf.items():
             new_node = doc.createElement(key)
-            new_node.appendChild(doc.createTextNode("""%s""" % eval('self.%s' % key)))
+            new_node.appendChild(doc.createTextNode(str(value)))
             root_node.appendChild(new_node)
         configuration = doc.toxml(encoding = 'utf-8')
         configuration += '\n' + root_node.toxml(encoding = 'utf-8')
         new_configuration_file = open(configuration_file, 'w')
         new_configuration_file.write(configuration)
         new_configuration_file.close()
+
+    temporary_dir = property(getTemporaryDir,
+                             setTemporaryDir)
+    text_fill = property(getTextFill,
+                         setTextFill)
+    image_fill = property(getImageFill,
+                          setImageFill)
+    boxes_stroke = property(getBoxesStroke,
+                            setBoxesStroke)
+    favorite_engine = property(getFavoriteEngine,
+                               setFavoriteEngine)
+    window_size = property(getWindowSize,
+                           setWindowSize)
+    unpaper = property(getUnpaper,
+                       setUnpaper)
+
+    unpaper_use_black_filter = property(getUseBlackFilter,
+                                        setUseBlackFilter)
+
+    unpaper_gray_filter_size = property(getGrayFilterSize,
+                                        setGrayFilterSize)
+
+    unpaper_noise_filter_intensity = property(getNoiseFilterIntensity,
+                                              setNoiseFilterIntensity)
+
+    unpaper_images_after_addition = property(getUnpaperImagesAfterAddition,
+                                             setUnpaperImagesAfterAddition)
+
+    unpaper_extra_options = property(getUnpaperExtraOptions,
+                                     setUnpaperExtraOptions)
+
+    improve_column_detection = property(getImproveColumnDetection,
+                                        setImproveColumnDetection)
+    column_min_width = property(getColumnMinWidth,
+                                setColumnMinWidth)
+
+    clean_text = property(getCleanText,
+                          setCleanText)
+
+    adjust_boxes_bounds = property(getAdjustBoxesBounds,
+                                   setAdjustBoxesBounds)
+
+    bounds_adjustment_size = property(getBoundsAdjustmentSize,
+                                      setBoundsAdjustmentSize)
+
+    deskew_images_after_addition = property(getDeskewImagesAfterAddition,
+                                            setDeskewImagesAfterAddition)
